'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useValue = exports.useLocal = exports.useQuery = exports.useDoc = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.useModel = useModel;

var _react = require('react');

var _Doc = require('../types/Doc');

var _Doc2 = _interopRequireDefault(_Doc);

var _Query = require('../types/Query');

var _Query2 = _interopRequireDefault(_Query);

var _QueryExtra = require('../types/QueryExtra');

var _QueryExtra2 = _interopRequireDefault(_QueryExtra);

var _Local = require('../types/Local');

var _Local2 = _interopRequireDefault(_Local);

var _Value = require('../types/Value');

var _Value2 = _interopRequireDefault(_Value);

var _subscribe = require('../subscribe');

var _subscriptionTypeFns = require('../subscriptionTypeFns');

var _model = require('@react-sharedb/model');

var _model2 = _interopRequireDefault(_model);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function useModel() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return (0, _react.useMemo)(function () {
    return _model2.default.scope.apply(_model2.default, args);
  }, [].concat(args));
}

var DEFAULT_COLLECTION = '$hooks';
var $collection = _model2.default.scope(DEFAULT_COLLECTION);

var useDoc = exports.useDoc = generateUseItemOfType(_subscriptionTypeFns.subDoc);
var useQuery = exports.useQuery = generateUseItemOfType(_subscriptionTypeFns.subQuery);
var useLocal = exports.useLocal = generateUseItemOfType(_subscriptionTypeFns.subLocal);
var useValue = exports.useValue = generateUseItemOfType(_subscriptionTypeFns.subValue);

function generateUseItemOfType(typeFn) {
  var isSync = typeFn === _subscriptionTypeFns.subLocal || typeFn === _subscriptionTypeFns.subValue;
  return function () {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    var $model = (0, _react.useMemo)(function () {
      return generateScopedModel();
    }, []);

    var hashedArgs = (0, _react.useMemo)(function () {
      return (0, _stringify2.default)(args);
    }, args);

    var cancelInitRef = (0, _react.useRef)();
    var destructorsRef = (0, _react.useRef)([]);
    var didMountRef = (0, _react.useRef)(false);
    var didInitRef = (0, _react.useRef)(false);

    useUnmount(function () {
      if (cancelInitRef.current) cancelInitRef.current.value = true;
      destructorsRef.current.forEach(function (destroy) {
        return destroy();
      });
      destructorsRef.current.length = 0;
      $model.destroy();
    });

    var params = (0, _react.useMemo)(function () {
      return typeFn.apply(undefined, args);
    }, [hashedArgs]);
    var collectionName = (0, _react.useMemo)(function () {
      return isQuery(params) ? getCollectionName(params) : undefined;
    }, [hashedArgs]);

    // For Query and QueryExtra return the scoped model targeting the actual collection path.
    // This is much more useful since you can use that use this returned model
    // to update items with: $queryCollection.at(itemId).set('title', 'FooBar')
    var $queryCollection = (0, _react.useMemo)(function () {
      return collectionName ? _model2.default.scope(collectionName) : undefined;
    }, [collectionName]);

    // TODO: Maybe enable returning array of ids for Query in future.
    //       See below for more info.
    // const idsName = useMemo(() => (
    //   hasIds(params) ? getIdsName($model.leaf()) : undefined
    // ), [])

    var initItem = (0, _react.useCallback)(function (params) {
      // Cancel previous initialization if it is active
      if (cancelInitRef.current) cancelInitRef.current.value = true;

      var item = getItemFromParams(params, $model.leaf());
      var destroySelf = function destroySelf() {
        item.unrefModel();
        item.destroy();
      };
      destructorsRef.current.push(destroySelf);

      var cancelInit = {};
      cancelInitRef.current = cancelInit;

      var finishInit = function finishInit() {
        // If another initialization happened while we were waiting for init to finish, don't do anything
        if (cancelInit.value) return;

        _subscribe.batching.batch(function () {
          // destroy the previous item and all unsuccessful item inits which happened until now.
          // Unsuccessful inits means the inits of those items which were cancelled, because
          // while the subscription was in process, another new item init started
          // (this might happen when the query parameter, like text search, changes quickly)
          // Don't destroy self though.
          destructorsRef.current.forEach(function (destroy) {
            return destroy !== destroySelf && destroy();
          });

          // Clear destructors array and add back self destroy
          destructorsRef.current.length = 0;
          destructorsRef.current.push(destroySelf);

          // Reference the new item data
          didInitRef.current = true;
          item.refModel();
        });
      };

      if (isSync) {
        finishInit();
      } else {
        item.init().then(finishInit);
      }
    }, []);

    // In case the data can be retrieved synchronously, get it right away
    if (isSync && !didInitRef.current) initItem(params);

    (0, _react.useLayoutEffect)(function () {
      // In case the result was retrieved synchronously, don't init again
      if (isSync && !didMountRef.current) {
        didMountRef.current = true;
        return;
      }
      initItem(params);
    }, [hashedArgs]);

    // In any situation force access data through the object key to let observer know that the data was accessed
    var data = $collection.get()[$model.leaf()];

    return [
    // Initialize async item as `null`
    // This way the strict `value === null` check can be used to determine
    // precisely whether the subscribe has finished executing
    isSync || didInitRef.current ? data : null,

    // Query, QueryExtra: return scoped model to collection path.
    // Everything else: return the 'hooks.<randomHookId>' scoped model.
    $queryCollection || $model

    // TODO: Maybe enable returning array of ids for Query in future.
    //       The potential drawback is that the rendering might fire twice
    //       when the subscribed data changes (items added or removed from query).
    //       This needs to be tested before enabling.
    // Return ids array as the third parameter for useQuery
    // idsName ? res.push($collection.get()[idsName]) : undefined
    ];
  };
}

function getCollectionName(params) {
  return params && params.params && params.params[0];
}

function isQuery(params) {
  var explicitType = params && params.__subscriptionType;
  return explicitType === 'Query' || explicitType === 'QueryExtra';
}

// TODO: Maybe enable returning array of ids for Query in future.
function hasIds(params) {
  var explicitType = params && params.__subscriptionType;
  return explicitType === 'Query';
}

function getItemFromParams(params, key) {
  var explicitType = params && params.__subscriptionType;
  var subscriptionParams = params.params;
  var constructor = getItemConstructor(explicitType);
  return new constructor($collection, key, subscriptionParams);
}

function getItemConstructor(type) {
  switch (type) {
    case 'Local':
      return _Local2.default;
    case 'Doc':
      return _Doc2.default;
    case 'Query':
      return _Query2.default;
    case 'QueryExtra':
      return _QueryExtra2.default;
    case 'Value':
      return _Value2.default;
    default:
      throw new Error('Unsupported subscription type: ' + type);
  }
}

function useUnmount(fn) {
  (0, _react.useLayoutEffect)(function () {
    return fn;
  }, []);
}

function generateScopedModel() {
  var path = DEFAULT_COLLECTION + '.' + _model2.default.id();
  return _model2.default.scope(path);
}