'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.subLocal = subLocal;
exports.subDoc = subDoc;
exports.subQuery = subQuery;
exports.subValue = subValue;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function subLocal(localPath) {
  if (typeof localPath !== 'string') {
    throw new Error('[react-sharedb] subLocal(): localPath must be a String. Got: ' + localPath);
  }
  return {
    __subscriptionType: 'Local',
    params: localPath
  };
}

function subDoc(collection, docId) {
  var invalid = void 0;
  if (typeof collection !== 'string') {
    throw new Error('[react-sharedb] subDoc(): `collection` must be a String. Got: ' + collection);
  }
  if (docId == null) {
    console.warn('\n      [react-sharedb] subDoc(): You are trying to subscribe to an undefined document id:\n        ' + collection + '.' + docId + '\n      Falling back to \'__NULL__\' document to prevent critical crash.\n      You should prevent situations when the `docId` is undefined.  \n    ');
    invalid = true;
  }
  if (invalid) docId = '__NULL__';
  return {
    __subscriptionType: 'Doc',
    __subscriptionInvalid: invalid,
    params: [collection, docId]
  };
}

function subQuery(collection, query) {
  var invalid = void 0;
  if (typeof collection !== 'string') {
    throw new Error('[react-sharedb] subQuery(): Collection must be String. Got: ' + collection);
  }
  if (query == null) {
    console.warn('\n      [react-sharedb] subQuery(): Query is undefined. Got:\n        ' + collection + ', ' + query + '\n      Falling back to {_id: \'__NON_EXISTENT__\'} query to prevent critical crash.\n      You should prevent situations when the `query` is undefined.  \n    ');
    invalid = true;
  }
  if (_lodash2.default.isString(query) || _lodash2.default.isArray(query) || _lodash2.default.isBoolean(query) || _lodash2.default.isNumber(query)) {
    throw new Error('\n      [react-sharedb] subQuery(): Query is not an Object. Got:\n        ' + collection + ', ' + query + '      \n      Query must always be an Object.\n    ');
  }
  if (invalid) query = { _id: '__NON_EXISTENT__' };
  return {
    __subscriptionType: (0, _util.isExtraQuery)(query) ? 'QueryExtra' : 'Query',
    __subscriptionInvalid: invalid,
    params: [collection, query]
  };
}

function subValue(value) {
  return {
    __subscriptionType: 'Value',
    params: value
  };
}